//------------------------------------------------------------------------
// Morph.java -- a simple engine for a graphical object world with
//               hierarchical struturing and layout management
//
// Andy Collins - CSE 590D - Spring 1997 - University of Washington
//------------------------------------------------------------------------
package application;

import java.awt.*;
import java.util.*;

class Morph extends Object
{
   private MorphPanel panel;

   protected Vector morphs;
   protected int baseHeight;
   protected int baseWidth;
   private Morph parent;
   protected Color fgColor;
   protected Color bgColor;
   protected static Color defaultFgColor = new Color (0,0,0);
   protected static Color defaultBgColor = new Color (240, 240, 240);

   static int shadowOffset = 3;
   static Color shadowColor = new Color(200, 200, 200);

   public boolean solid;
   public boolean outline;

   public Morph (int xv, int yv, int zv, int w, int h, Color bg, Color fg)
   {
      panel = null;

      parent = null;
      morphs = new Vector ();
      x = xv;
      y = yv;
      z = zv;
      baseHeight = h;
      baseWidth = w;
      Layout ();

      solid = true;
      outline = false;

      bgColor = bg;
      fgColor = fg;

      
   }

   public Morph ()
   {
      this (0, 0, 0, 10, 10, defaultFgColor, defaultFgColor);
   }

   public void SetPanel (MorphPanel p)
   {
      panel = p;
   }

   public MorphPanel GetPanel ()
   {
      if (panel != null)
         return panel;
      else if (parent != null)
         return parent.GetPanel ();
      else
         return null;
   }

   public void SetFgColor (Color c)
   {
      fgColor = c;
   }

   public void SetBgColor (Color c)
   {
      bgColor = c;
   }

   public void SetHeight (int h)
   {
      baseHeight = h;
      Layout ();
   }
   public void SetWidth (int w)
   {
      baseWidth = w;
      Layout ();
   }

   public void AddMorph (Morph m)
   {
      morphs.addElement (m);
      m.parent = this;
      Layout ();
   }

   public void RemoveMorph (Morph m)
   {
      morphs.removeElement (m);
      Layout ();
   }

   private int x;
   private int y;
   private int z;

   public void SetZ (int zv)
   {
      z = zv;
      
   }
   public void MoveTo (int xv, int yv)
   {
      x = xv;
      y = yv;
   }
   public int GetX ()
   {
      return x;
   }
   public int GetY ()
   {
      return y;
   }
   public int GetZ ()
   {
      return z;
   }

   public int GetGlobalX ()
   {
      if (parent != null)
         return (GetX () + parent.GetGlobalX ());
      else
         return GetX ();
   }
   public int GetGlobalY ()
   {
      if (parent != null)
         return (GetY () + parent.GetGlobalY ());
      else
         return GetY ();
   }

   public boolean IsOver (int xv, int yv)
   {
      boolean retval = false;

      if (!solid)
         retval = false;
      else
         retval = ((xv > x) && (yv > y) && (xv < x + width) && (yv < y + height));

      Enumeration e = morphs.elements ();
      while (e.hasMoreElements ())
         if ( ((Morph) e.nextElement ()).IsOver (xv-x, yv-y) )
            retval = true;

      return retval;
   }

   protected int width;
   protected int height;

   public void Layout ()
   {
      Enumeration e = morphs.elements ();
      while (e.hasMoreElements ())
         ((Morph) e.nextElement ()).Layout ();

      width = baseWidth;
      height = baseHeight;
   }
   public int GetHeight ()
   {
      return height;
   }
   public int GetWidth ()
   {
      return width;
   }

   public void Paint (Graphics g, int xoff, int yoff, Color c, boolean selected)
   {
      Enumeration e;

      if (selected)
         {
         PaintBase (g, xoff+shadowOffset, yoff+shadowOffset, shadowColor, shadowColor);
         e = morphs.elements ();
         while (e.hasMoreElements ())
            ((Morph) e.nextElement ()).Paint (g, xoff + x, yoff + y, bgColor, true);
         }

      PaintBase (g, xoff, yoff, bgColor, fgColor); 
      e = morphs.elements ();
      while (e.hasMoreElements ())
         ((Morph) e.nextElement ()).Paint (g, xoff + x, yoff + y, c, false);
   }

   public void PaintBase (Graphics g, int xoff, int yoff, Color bgColor, Color fgColor)
   {
      if (solid)
         {
         g.setColor (bgColor);
         g.fillRect (xoff + x, yoff + y, width, height);
         }

      if (outline)
         {
         g.setColor (fgColor);
         g.drawRect (xoff + x, yoff + y, width, height);
         }
   }
}

class LabelMorph extends Morph
{
   private boolean textSolid;

   private String label;
   public LabelMorph (String l)
   {
      label = l;
      SetHeight (15);
      SetWidth (50);
      textSolid = false;
   }

   public void PaintBase (Graphics g, int xoff, int yoff, Color bgColor, Color fgColor)
   {
      super.PaintBase (g, xoff, yoff, bgColor, fgColor);

      Font font = new Font ("TimesRoman", Font.PLAIN, 12);
      FontMetrics fm = g.getFontMetrics (font);

      if (textSolid && !solid)
         {
         g.setColor (bgColor);
         g.fillRect (xoff + GetX (), yoff + GetY (), fm.stringWidth (label), height);
         }

      g.setColor (fgColor);
      g.drawString (label, xoff + GetX (), yoff + GetY() + GetHeight () - 2);
   }
}

class ColumnMorph extends Morph
{
   private int inset;
   public void SetInset (int i)
   {
      inset = i;
   }

   public void Layout ()
   {
      super.Layout ();

      height = 0;
      width = baseWidth;
      Enumeration e;

      e = morphs.elements ();
      while (e.hasMoreElements ())
         {
         Morph m = (Morph) e.nextElement ();
         height += m.GetHeight ();
         if (m.GetWidth () > width)
            width = m.GetWidth ();
         }

      if (height < baseHeight)
         height = baseHeight;

      height += 2*inset;
      width += 2*inset;

      int curY = inset;
      e = morphs.elements ();
      while (e.hasMoreElements ())
         {
         Morph m = (Morph) e.nextElement ();
         m.MoveTo (inset, curY);
         curY += m.GetHeight (); 
         }
   }

   public void PaintBase (Graphics g, int xoff, int yoff, Color bgColor, Color fgColor)
   {
      super.PaintBase (g, xoff, yoff, bgColor, fgColor);
//      g.drawRect (xoff + x, yoff + y, width, height);
   }

}











